/* osinfo.c
 * ========
 *
 * ver 1.00
 *
 *		by Sessai@NIPR
 */

#ifdef __USAGE
%C - print QNX OS info on this node etc
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/name.h>

#include <sys/timers.h>	/* necessary for getclock function!!! */
#include <time.h>

/* for clock_report() !!!! */
#include <signal.h>
#include <sys/name.h>
#include <errno.h>
#include <sys/vc.h>
#include <sys/kernel.h>
/* #include <sys/sendmx.h>	*/
#include <sys/osinfo.h>

void qnx_now(int *yr,int *mon,int *day,int *hour,int *min,int *sec,
		long *tnsec,int *tday) {
	time_t qnx_time;
	struct tm *gmt_time;
	struct timespec time_spec;
	/* int msec,usec,nsec; */

	/* qnx_time = time(NULL); */
	getclock(TIMEOFDAY,&time_spec);
	qnx_time = (time_t)time_spec.tv_sec;
	gmt_time = gmtime(&qnx_time);
	*yr   = gmt_time->tm_year + 1900;
	*mon  = gmt_time->tm_mon + 1;
	*day  = gmt_time->tm_mday;
	*hour = gmt_time->tm_hour;
	*min  = gmt_time->tm_min;
	*sec  = gmt_time->tm_sec;
	*tday = gmt_time->tm_yday + 1;	/* 1/1 = 1 */
	*tnsec = time_spec.tv_nsec;
	//	*msec = (int)(time_spec.tv_nsec/1000000L);
	//	*usec = (int)((time_spec.tv_nsec/1000L)%1000L);
	//	*nsec = (int)(time_spec.tv_nsec%1000000L);
	return;
}

int remote_qnx_now(int node,int *yr,int *mon,int *day,
		int *hour,int *min,int *sec,long *tnsec,int *tday) {
	nid_t nid;
	pid_t procpid;

	time_t qnx_time;
	struct tm *gmt_time;
	struct timespec time_spec;
	/* int msec,usec,nsec; */

	nid=(nid_t)node;
	if( (procpid = qnx_vc_attach(nid,PROC_PID,0,_VC_AT_SHARE) )==-1){
		perror("qnx_vc_attach");
		return -1;
	}

	/* qnx_time = time(NULL); */
	// getclock(TIMEOFDAY,&time_spec);
	if (qnx_getclock(procpid,TIMEOFDAY,&time_spec)==-1){
		perror("qnx_get_clock");
		qnx_vc_detach(procpid);
		return -2;
	}
		qnx_vc_detach(procpid);

	qnx_time = (time_t)time_spec.tv_sec;
	gmt_time = gmtime(&qnx_time);
	*yr   = gmt_time->tm_year + 1900;
	*mon  = gmt_time->tm_mon + 1;
	*day  = gmt_time->tm_mday;
	*hour = gmt_time->tm_hour;
	*min  = gmt_time->tm_min;
	*sec  = gmt_time->tm_sec;
	*tday = gmt_time->tm_yday + 1;	/* 1/1 = 1 */
	*tnsec = time_spec.tv_nsec;
	/*	msec = (int)(time_spec.tv_nsec/1000000L);
		usec = (int)((time_spec.tv_nsec/1000L)%1000L);
		nsec = (int)(time_spec.tv_nsec%1000000L);
	 */
	return 0;
}

int doy(int *yr,int *mon,int *day){

	/* calculate day of the year and return it.
	 *	1/1=1
	 */
	int tday,leap;

	if(*yr%4==0)leap=1;
	else        leap=0;

	if(*mon== 1)      tday=*day;
	else if(*mon== 2) tday=*day     + 31;
	else if(*mon== 3) tday=*day+leap+ 59;
	else if(*mon== 4) tday=*day+leap+ 90;
	else if(*mon== 5) tday=*day+leap+120;
	else if(*mon== 6) tday=*day+leap+151;
	else if(*mon== 7) tday=*day+leap+181;
	else if(*mon== 8) tday=*day+leap+212;
	else if(*mon== 9) tday=*day+leap+243;
	else if(*mon==10) tday=*day+leap+273;
	else if(*mon==11) tday=*day+leap+304;
	else if(*mon==12) tday=*day+leap+334;
	else		  tday= -1;

	return tday;
}

/* ------------------------------------------------------------ */

int clock_report(FILE *fp) {

	int yr,mon,day,hr,min,sec,tday,msec,usec;
	int yr1,mon1,day1,hr1,min1,sec1,tday1;
	int yr2,mon2,day2,hr2,min2,sec2,tday2;
	long tnsec,tnsec1,tnsec2;

	/* time_t qnx_time; */
	/* struct tm *gmt_time; */
	/* struct timespec time_spec; */

	int i,flg,num_nids,n;

	static char alive[32];

	pid_t pid,vid;
	pid_t procpid;
	nid_t nid;
	nid_t nids[32];

	struct _osinfo osdata;


//	fprintf(fp,"OSinfo & remote clock Report .....................\n");

   /* OSinfo... */

	if(qnx_osinfo(0,&osdata) != -1){
		fprintf(fp,"Node %ld : ",osdata.nodename);
		fprintf(fp,"QNX Ver %d  ",osdata.version);
		fprintf(fp,"Rel %c\n",osdata.release);

		fprintf(fp,"...PC: %s, ",osdata.machine);
		fprintf(fp,"Boot: %c, ",osdata.bootsrc);
		fprintf(fp,"Cpu/Fpu: %ld/%ld, ",osdata.cpu,osdata.fpu);
		fprintf(fp,"CpuSpeed: %d, ",osdata.cpu_speed/96);
		fprintf(fp,"Tick(usec): %d\n",osdata.tick_size);

		fprintf(fp,"...Free/TotalMem: %d/%dk, ",
					osdata.freememk,osdata.totmemk);
		fprintf(fp,"MaxProcNum: %d, ",osdata.num_procs);
		fprintf(fp,"SysFlag: %x, ",osdata.sflags);
	}


   /* This node is ... */

	/* For this to work, 'export NODE' is required before running me.
	 * If 'NODE' is not exported, n will be 0...
	 * 	if( (n=atoi(getenv("NODE"))) != 0)
	 *		fprintf(fp,"This is node %d ...\n",n);
	 */
	/* This works fine at any time! */
	// fprintf(fp,"CLK This is node %ld ...\n",osdata.nodename);

   /* Time Zone of this node... */

	fprintf(fp,"TZ: %s ...",getenv("TZ"));

	fprintf(fp,"\n");

   /* alive -u .... */

	if((num_nids=qnx_net_alive(alive,32))==-1){
		fprintf(fp,"qnx_net_alive() failed\n");
	}
	/*
		fprintf(fp,"  alive node(s) : ");
	// for(i=1;i<=num_nids;i++){
	for(i=1;i<num_nids;i++){
		if(alive[i]) fprintf(fp,"%2d ",i);
	}
		fprintf(fp,"\n");
	*/

   /* This node is ... AGAIN!!! */

	for(i=1;i<num_nids;i++){
		if(alive[i]){
			nid=(nid_t)i;
		   if((procpid=qnx_vc_attach(nid,PROC_PID,0,_VC_AT_SHARE))==-1){
			fprintf(fp,"CLK qnx_vc_attach() failed\n");
		   }
			qnx_vc_detach(procpid);
		   if(procpid == PROC_PID){
			if(i!=osdata.nodename){
			    fprintf(fp,"CLK This is node %d ...",i);
			    fprintf(fp," ... Aryaaa?!?!?!...\n");
			}
		   }
		}
	}

   /* print `alive -u` here.... */

		fprintf(fp,"  alive node(s) : ");
	// for(i=1;i<=num_nids;i++){
	for(i=1;i<num_nids;i++){
		if(alive[i]) fprintf(fp,"%2d ",i);
	}
		fprintf(fp,"\n");

   /* nameloc ... */

		fprintf(fp,"Name Locator(s) : ");
	for(i=0;i<qnx_name_locators(&nids);++i){
		fprintf(fp,"%2ld ",nids[i]);
	}
		fprintf(fp,"\n");

#if 0
   /* OS clocks on alive nodes... */

	for(n=1;n<num_nids;n++){
	    if(alive[n]){
		remote_qnx_now(n,&yr1,&mon1,&day1,
				&hr1,&min1,&sec1,&tnsec1,&tday1);
		fprintf(fp,
		    "node %d QNX %d/%2d/%2d (%03d) %2d:%02d:%02d.%09ld\n",
				n,yr1,mon1,day1,tday1,hr1,min1,sec1,tnsec1);
	    }
	}
#endif

	return 0;
}



void main(){
	clock_report(stdout);
	exit(0);
}
