/* wss.c
   -----
	wss - replacemant of old 'ws' command on QNX4.22
		for Y2K compliance etc (for QNX4.24 or later)
					in autumn by Sessai@NIPR

	Differences:
		spcify 4-digit year!!! not 2-digit!!!
		no interactive mode (-i/-e)
		verbose mode (instead of -q) including detail file info display
		by default all files and directories will be processed
		except "." and "..". "-d" option exists and "-f" option added
		only check modification time(not ftime/ctime/atime but mtime!)
		no -m/-u/-g option
		no pattern matching (-p/-P option)
		no recursive oprtation and -l option
		don't care TimeZone...
	ToDo:
		recursive operation
		pattern matching
		-e ignore|prompt|abort
		prefix handling is not always good!
 */

/* original ws command (on QNX 4.22) usage
   ---------------------------------------
ws - walk directory structure executing a command (QNX)

*** Note - it is recommended that you use the find utility
*** instead of ws.

ws [options]... [[directory] "command"]
Options:
 -a date/time               Use only files after date/time.
 -b date/time               Use only files before date/time.
 -c [p=c|d=c]               Set path/delete char to character c or \\octal.
 -d                         Exec on directories, not files.
 -e ignore|prompt|abort     On an error; ignore, prompt or abort.
 -g group[,group]*          Use only files belonging to these groups.
 -l levels                  Recurse this number of levels.
 -m mode[,mode]*            Use only on files matching these modes.
 -[Pp] pattern[,pattern]*   All files exec'ed on must match this pattern.
 -q                         Quiet. Don't display commands executed.
 -i                         Interactive. Query user before executing
                            each invocation of 'command'.
 -r                         Pass relative paths to commands.
 -t                         Ignore the date/time of files being processed.
 -u owner[,owner]*          Exec on files belonging to these users.
Where:
 Date/time format is "[UTC] day/month/yr hr:min:sec", 24 hr clock.
 Mode format is octal or <[u|g|o]...|a>=[r|w|s]...
 Modes and patterns may be preceded by '!' meaning 'not'.
*/

#ifdef __USAGE
%C - do something strange...
	as a replacement of the old non-y2k-compliant "ws" command in qnx422
							by Sessai@NIPR in 1999
%C [-v|-vv] [-w|-ww|-www] [-f|-F|-d|-D|-L|-S] [-r]
	[-a YYYY MM DD hh mm ss] [-b YYYY MM DD hh mm ss] [dir [cmd]]

	-v   : show each file name before executing command
	-vv  : show each file info before executing command
			"ws dir" = "ws -v dir" = "ls -1 dir"
			"ws -vv dir" is similar to "ls -laF dir"
		 the file info format is
		     YYYY/MM/DD hh:mm:ss TMode n u/g size name[c [-> lname[c]]]
			MTime TypeMode Nlink Uid/Gid Size Name
			Type :
				- : regular file                  (8)
				d : directory                     (4)
				l : symlink                       (A)
				b : block special device file     (6)
				c : character special device file (2)
				n : FIFO/Socket/SpecialNamedFile  (1/C/5)
				X : unknown(=(stat.st_mode&0F0000)>>12)
			code :
				/ : directory
				@ : symlink
				# : FIFO/Socket/SpecialNamedFile
				??: unknown
	-w   : show command error code if non zero
		(error will be put in stderr.)
	-ww  : show command string and command error code if non zero
		(command string will be put in stdout)
	-www : show what -ww shows and more info
		(more info will also be put in stdout)
	-f : invoke command only to regular files (excluding symlinks!!)
		 (do nothing to any directories, symlinks, special files etc.)
	-F : invoke command only to regular files including symlink files!
		 (do nothing to any directories, special files etc.)
		 (but do action for a symlink file even if it points to dir.)
	-d : invoke command only to directories (NOT including symlink!)
	         ("." and ".." will always be skipped!!!)
		 (do nothing to any regular files, links, special files etc.)
	-D : invole command only to any files except directories
		 (do action for a symlink file even if it points to dir.)
		 (try to do action also for any special files.)
		 (This was the default behavior of old "ws" command.)
	-L : only symlink files will be processed. Others will just be skipped.
	-S : only special files will be processed. Others will just be skipped.
	* if neither -f,-F nor -d,-D,-L,-S specified,
		all files except "." and ".." will be processed!
	-r : Pass relative paths(only file name (not include dir name)
	       to commands (even if DIR is an absolute path(dir) name).
	        (if DIR is an absolute path (starting with '/'),
		  @ char in cmd string will be replaced by abs_path by default.
	          but -r specified, rel_path(only filename) will be passed.
		 if DIR is a relative path, no effect.
		 chdir(DIR) is always done before each command will be invoked.)
		DIR is AbsPath and -r not specified:
			chdir(DIR) and @ will be replaced by AbsolutePathName
		DIR is AbsPath and -r specified:
			chdir(DIR) and @ will be replaced by filename(no dir)
		DIR is RelPath and -r not specified:
			chdir(DIR) and @ will be replaced by filename(no dir)
		DIR is RelPath and -r specified:
			chdir(DIR) and @ will be replaced by filename(no dir)
	-a ... : invoke command  only to files
	                whose mtime are "equal or after" specified time
	-b ... : invoke command  only to files
	                whose mtime are "before" specified time
	dir    : directory where this program will apply
	         if omitted, "." will be assumed.
		 BUT you MUST specify this if you specify command below!!!
	cmd    : command string which will operate to each file
	         '@' character will be replaced by each file name.
	         if omitted, "echo @" will be assumed.
		 (but if -v or -vv specified, NULL string will be assumed.)
	         quotation marks are required for strings including spaces

	YYYY : year : This must be a 4 digit year!!!!
	 MM  : month
	 DD  : day
	 hh  : hour
	 mm  : minute
	 ss  : second

    [-v|-vv] [-w|-ww|-www] [-f|-F|-d|-D|-L|-S] [-r] [-a ..] [-b ..] [dir [cmd]]
		MUST be ordered like this!!!

	* "." and ".." will always be skipped.
	* output is not sorted (like original ws)!!!
	* no recursive operation so far!!!
					(sorry for no flexibility!)

	* "wss" = "wss ." = "wss . "echo @"" = "wss -v" ="ls -1"
	* "wss -vv" is similar to "ls -laF"

  Example1 (under /bin/ksh)
       ws -a DD/MM/YY hh:mm:ss -b "DD/MM/YY hh:mm:ss" -q DIR "cp -p @ DIR2"
    is equivalent to
       wss -D -a YYYY MM DD hh mm ss -b YYYY MM DD hh mm ss DIR "cp -p @ DIR2"
  Example2 (under /bin/ksh)
       ws -a "DD/MM/YY hh:mm:ss" -q -r DIR "echo @"
    is equivalent to
       wss -D -r -a "DD/MM/YY hh:mm:ss" DIR ["echo @"]
  Example3
       ws -d -q -r DIR "ls -laFd @"
    is equivalent to
       wss -d -r DIR "ls -laFd @"
  Example4
    show files which were modified yesterday
    under current directory (but not recursively)
    (and sort in time order...)
       wss [-r] -vv `yesterday` 0 0 0 -b `today` 0 0 0 . | sort
  Example5
    show only regular files which were modified these 3 hours in ./ dir
       wss [-r] -f -a $(ymdhmsnow -h 3) -b `ymdhmsnow` [. ["echo @"]]
  Example6
    print file list in current directory
       wss
  Example7
    show all files' info in detail in DIR directory
       wss -vv DIR
#endif

#include <stdlib.h>	// for _MAX_NODE etc
#include <stdio.h>
#include <dirent.h>	//for [open|read|close]dir
#include <unistd.h>	//for access();
#include <sys/io_msg.h>	// for _QNX_PATH_MAX
#include <string.h>
#include <time.h>	//for time_t,struct tm,gmtime();
#include <sys/wait.h>	//for system();

char *rep_at_file(char *orgcmd,char keychr,char *rep);

void usage(argc,argv)
	int argc;
	char *argv[];
{
	char cmd[48];
	strcpy(cmd,"use ");
	strcat(cmd,argv[0]);
	strcat(cmd," 1>&2");
	system(cmd);
	exit(-1);
}

void main(argc,argv)
	int argc;
	char *argv[];
{
	DIR *dirp;
	struct dirent *direntp;

	//char fullpath[256]="";
	char fullpath[_QNX_PATH_MAX]="";
	char file[_QNX_PATH_MAX]="";
	struct stat statbuf;
	struct stat statdirbuf;
	struct stat statlbuf;
	//char pwd[_QNX_PATH_MAX]="";
	char *pwd;

	int linklen=0;
	char linkname[PATH_MAX + 1];
	char lfile[_QNX_PATH_MAX]="";
	int link=0;
	int reg=0;
	int dir=0;
	int special=0;
	int device=0;
	int unknown=0;
	int xxx=0;
	int llink=0;
	int lreg=0;
	int ldir=0;
	int lspecial=0;
	int ldevice=0;
	int lunknown=0;

	time_t t;
	struct tm tms;
	struct tm atms;
	struct tm btms;

	int optflag=0;
	int v=0;
	int w=0;
	int after=0;
	int before=0;
	unsigned long aut=0L,but=0L;

	int ayr=0,amo=0,ady=0,ahr=0,amn=0,asc=0;
	int byr=0,bmo=0,bdy=0,bhr=0,bmn=0,bsc=0;

	int ok=0;
	int regonly=0;
	int regandlinkonly=0;
	int dironly=0;
	int exceptdir=0;
	int linkonly=0;
	int specialonly=0;

	char cmd[1024];
	char realcmd[1024]="";
	char keychr='@';
	int rc,rc2;

	int absolute_path=0;
	int rel=0;

#if 0
	char node[_MAX_NODE];
	char dir[_MAX_DIR];
	char fn[_MAX_FNAME];
	char ext[_MAX_EXT];
	char ndir[_MAX_NODE+_MAX_DIR];
	char fnext[_MAX_FNAME+_MAX_EXT];
#endif

#if 1
	if(w>2)	fprintf(stderr,
		"_QNX_PATH_MAX=%d,_MAX_NODE=%d,_MAX_DIR=%d,_MAX_FNAME=%d,_MAX_EXT=%d\n",
		_QNX_PATH_MAX,_MAX_NODE,_MAX_DIR,_MAX_FNAME,_MAX_EXT);
fflush(stderr);
#endif

	//if(argc==1)usage(argc,argv);
	//if(argc<=1+optflag)usage(argc,argv);
	if(argc>1+optflag){
		//if((strcmp(argv[1+optflag],"-l")==0)||(strcmp(argv[1+optflag],"-v")==0)){..}
		if(strcmp(argv[1+optflag],"-v")==0){
			v=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-vv")==0){
			v=2;
			optflag++;
		}
	}
	if(argc>1+optflag){
		if(strcmp(argv[1+optflag],"-w")==0){
			w=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-ww")==0){
			w=2;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-www")==0){
			w=3;
			optflag++;
		}
	}
	//if(argc<=1+optflag)usage(argc,argv);
	if(argc>1+optflag){
		if((strcmp(argv[1+optflag],"-l")==0)||(strcmp(argv[1+optflag],"-v")==0)){
			v++;
			optflag++;
		}
	}
	if(argc>1+optflag){
		if((strcmp(argv[1+optflag],"-l")==0)||(strcmp(argv[1+optflag],"-v")==0)){
			v++;
			optflag++;
		}
	}
	if(argc>1+optflag){
		if((strcmp(argv[1+optflag],"-l")==0)||(strcmp(argv[1+optflag],"-v")==0)){
			v++;
			optflag++;
		}
	}
#if 0
	if(argc>1+optflag){
		//if(strcmp(argv[1+optflag],"-F")==0){...}
		if((strcmp(argv[1+optflag],"-F")==0)||(strcmp(argv[1+optflag],"-f")==0)){
			regonly=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-d")==0){
			dironly=1;
			optflag++;
		}
	}
#else
	if(argc>1+optflag){
		//if((strcmp(argv[1+optflag],"-F")==0)||(strcmp(argv[1+optflag],"-f")==0)){...}
		if(strcmp(argv[1+optflag],"-f")==0){
			regonly=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-F")==0){
			regandlinkonly=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-d")==0){
			dironly=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-D")==0){
			exceptdir=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-L")==0){
			linkonly=1;
			optflag++;
		} else if(strcmp(argv[1+optflag],"-S")==0){
			specialonly=1;
			optflag++;
		}
	}
#endif
	if(argc>1+optflag){
		if(strcmp(argv[1+optflag],"-r")==0){
			rel=1;
			optflag++;
		}
	}
	if(argc>1+optflag){
		if(strcmp(argv[1+optflag],"-a")==0){
			after=1;
			if(argc<=1+optflag+7)usage(argc,argv);
			ayr=atoi(argv[1+optflag+1]);
			amo=atoi(argv[1+optflag+2]);
			ady=atoi(argv[1+optflag+3]);
			ahr=atoi(argv[1+optflag+4]);
			amn=atoi(argv[1+optflag+5]);
			asc=atoi(argv[1+optflag+6]);
			optflag+=7;
			atms.tm_year=ayr-1900;
			atms.tm_mon=amo-1;
			atms.tm_mday=ady;
			atms.tm_hour=ahr;
			atms.tm_min=amn;
			atms.tm_sec=asc;
			atms.tm_isdst=0;
			aut=(unsigned long)mktime(&atms);
		}
	}
	if(argc>1+optflag){
		if(strcmp(argv[1+optflag],"-b")==0){
			before=1;
			if(argc<=1+optflag+7)usage(argc,argv);
			byr=atoi(argv[1+optflag+1]);
			bmo=atoi(argv[1+optflag+2]);
			bdy=atoi(argv[1+optflag+3]);
			bhr=atoi(argv[1+optflag+4]);
			bmn=atoi(argv[1+optflag+5]);
			bsc=atoi(argv[1+optflag+6]);
			optflag+=7;
			btms.tm_year=byr-1900;
			btms.tm_mon=bmo-1;
			btms.tm_mday=bdy;
			btms.tm_hour=bhr;
			btms.tm_min=bmn;
			btms.tm_sec=bsc;
			btms.tm_isdst=0;
			but=(unsigned long)mktime(&btms);
		}
	}
	if(argc<=1+optflag){
		strcpy(fullpath,".");
		//strcpy(cmd,"echo @");
		if(v==0){
			strcpy(cmd,"echo @");
		} else {
			strcpy(cmd,"");
		}
	}else{
		strcpy(fullpath,argv[1+optflag]);
		optflag++;
		if(argc<=1+optflag){
			//strcpy(cmd,"echo @");
			if(v==0){
				strcpy(cmd,"echo @");
			} else {
				strcpy(cmd,"");
			}
		} else {
			strcpy(cmd,argv[1+optflag]);
			optflag++;
		}
	}

	if(fullpath[0]=='/')absolute_path=1;
	if(rel)absolute_path=0;


	//XXXXX//pwd=getenv("PWD");
	pwd=getcwd(NULL,0);
	if(pwd==NULL){
		fprintf(stderr,"Cannot get current directory\n");
		exit(1);
	}

	if(w>2){
		int i;
		fprintf(stderr,"cmdline :");
		for(i=0;i<argc;i++){
			fprintf(stderr," %s",argv[i]);
		}
		fprintf(stderr,"\n");
		fprintf(stderr,"argv0=%s\n",argv[0]);
		fprintf(stderr,"verbose=%d\n",v);
		fprintf(stderr,"wss-verbose=%d\n",w);
		fprintf(stderr,"after=%d\n",after);
		fprintf(stderr,"adate=%04d/%02d/%02d %02d:%02d:%02d (%lu)\n",
				ayr,amo,ady,ahr,amn,asc,aut);
		fprintf(stderr,"before=%d\n",before);
		fprintf(stderr,"bdate=%04d/%02d/%02d %02d:%02d:%02d (%lu)\n",
				byr,bmo,bdy,bhr,bmn,bsc,but);
		fprintf(stderr,"DIR=%s\n",fullpath);
		fprintf(stderr,"(absolute_path=%d)\n",absolute_path);
		fprintf(stderr,"cmd=%s\n",cmd);
		fprintf(stderr,"PWD=%s\n",pwd);
fflush(stdout);
fflush(stderr);
	}
	//exit(0);

		if((strcmp(fullpath,"")==0)){
			fprintf(stderr,"filename is NULL, exit...\n");
			exit(-1);
		}
		if(access(fullpath,R_OK)==-1){
			fprintf(stderr,
			"the file %s does not exist or unreadable, exit...\n",
				fullpath);
			exit(-1);
		}
		//fprintf(stdout,"Yahho fullpath=%s\n",fullpath);

#if 0
	_splitpath(fullpath,node,dir,fn,ext);
		fprintf(stdout,"%s : NODE%s DIR%s FN%s EXT%s ...\n",
				fullpath,node,dir,fn,ext);

	strcpy(ndir,node);strcat(ndir,dir);
	if(strcmp(ndir,"")==0)strcpy(ndir,"./");
	strcpy(fnext,fn);strcat(fnext,ext);
		fprintf(stdout,"NDIR%s FNEXT%s ...\n",ndir,fnext);
	if(access(ndir,R_OK)==-1){
		fprintf(stderr,"the dir %s does not exist or unreadable, exit...\n",ndir);
		exit(-1);
	}
	dirp=opendir(ndir);
	if(dirp==NULL){
		fprintf(stderr,"opendir %s FAILED! exit\n",ndir);
		exit(-1);
	}
	closedir(dirp);
		//fprintf(stdout,"Yahho OK NDIR%s FNEXT%s ...\n",ndir,fnext);
#endif

	if(access(fullpath,R_OK)==-1){
		fprintf(stderr,"the dir %s does not exist or unreadable, exit...\n",fullpath);
		exit(-1);
	}
	dirp=opendir(fullpath);
	if(dirp==NULL){
		fprintf(stderr,"opendir %s FAILED! exit\n",fullpath);
		exit(-1);
	}
		if(stat(fullpath,&statdirbuf)!=0){
			perror("stat:");
			fprintf(stderr,"stat %s failed\n",fullpath);
			exit(-1);
		}
#if 0
		fprintf(stdout,"st_ino=%ld\n",statdirbuf.st_ino);
		fprintf(stdout,"st_dev=%ld\n",statdirbuf.st_dev);
		fprintf(stdout,"st_size=%ld\n",statdirbuf.st_size);
		fprintf(stdout,"st_rdev=%ld\n",statdirbuf.st_rdev);
		fprintf(stdout,"st_uid=%d\n",statdirbuf.st_uid);
		fprintf(stdout,"st_gid=%d\n",statdirbuf.st_gid);
		fprintf(stdout,"st_ftime=%ld\n",statdirbuf.st_ftime);
		fprintf(stdout,"st_mtime=%ld\n",statdirbuf.st_mtime);
		fprintf(stdout,"st_atime=%ld\n",statdirbuf.st_atime);
		fprintf(stdout,"st_ctime=%ld\n",statdirbuf.st_ctime);

		fprintf(stdout,"st_mode=%04X\n",statdirbuf.st_mode);

		fprintf(stdout,"st_modeX=%1X\n",(statdirbuf.st_mode&0xF000)>>12);
		fprintf(stdout,"st_modeXO=%04o\n",(statdirbuf.st_mode&0007777));

		fprintf(stdout,"st_ouid=%d\n",statdirbuf.st_ouid);
		fprintf(stdout,"st_ogid=%d\n",statdirbuf.st_ogid);
		fprintf(stdout,"st_nlink=%u\n",statdirbuf.st_nlink);
		fprintf(stdout,"st_status=%04X\n",statdirbuf.st_status);
#endif

	//XXXXX//chdir(fullpath);

	while(1){
		if((direntp=readdir(dirp))==NULL)break;

		//fprintf(stdout,"%s\n",direntp->d_name);

		if(strcmp(direntp->d_name,".")==0)continue;
		if(strcmp(direntp->d_name,"..")==0)continue;

			xxx=0;
		if(S_ISDIR(direntp->d_stat.st_mode)){
			//fprintf(stdout,"d");
		} else if(S_ISREG(direntp->d_stat.st_mode)){
			//fprintf(stdout,"-");
		} else if(S_ISBLK(direntp->d_stat.st_mode)){
			//fprintf(stdout,"b");
		} else if(S_ISCHR(direntp->d_stat.st_mode)){
			//fprintf(stdout,"c");
		} else if(S_ISLNK(direntp->d_stat.st_mode)){
			//fprintf(stdout,"l");
		} else if(S_ISFIFO(direntp->d_stat.st_mode)){
			//fprintf(stdout,"F");
		} else if(S_ISSOCK(direntp->d_stat.st_mode)){
			//fprintf(stdout,"S");
		} else if(S_ISNAM(direntp->d_stat.st_mode)){
			//fprintf(stdout,"N");
		} else {
			xxx=1;
			////fprintf(stdout,"X");
			//fprintf(stdout,"%1X",(direntp->d_stat.st_mode&0xF000)>>12);
		}
			//fprintf(stdout," ");

#if 0
		if(v>4){
			//fprintf(stdout,"%04X",direntp->d_stat.st_mode);
			fprintf(stdout,"%1X",(direntp->d_stat.st_mode&0xF000)>>12);
			fprintf(stdout,"%04o",(direntp->d_stat.st_mode&0007777));
			fprintf(stdout," ");
		}
#endif

		strcpy(file,fullpath);
		if(fullpath[strlen(fullpath)-1]!='/')strcat(file,"/");
		strcat(file,direntp->d_name);

		//if(stat(file,&statbuf)!=0){...}
		if(lstat(file,&statbuf)!=0){
			perror("stat:");
			fprintf(stderr,"stat %s failed\n",file);
			exit(-1);
		}

	if((S_ISREG(statbuf.st_mode)==0)&&(regonly))continue;
	if((S_ISREG(statbuf.st_mode)==0)&&(S_ISLNK(statbuf.st_mode)==0)&&(regandlinkonly))continue;
	if((S_ISDIR(statbuf.st_mode)==0)&&(dironly))continue;
	if((S_ISDIR(statbuf.st_mode))&&(exceptdir))continue;
	if((S_ISLNK(statbuf.st_mode)==0)&&(linkonly))continue;
	if((S_ISBLK(statbuf.st_mode)==0)&&(S_ISCHR(statbuf.st_mode)==0)
		&&(S_ISFIFO(statbuf.st_mode)==0)&&(S_ISSOCK(statbuf.st_mode)==0)
		&&(S_ISNAM(statbuf.st_mode)==0)&&(specialonly))continue;

	ok=0;
	if(after||before){
		if((after)&&(before==0)){
			//if((unsigned long)statbuf.st_mtime>aut)ok=1;
			if((unsigned long)statbuf.st_mtime>=aut)ok=1;
		} else if((after==0)&&(before)){
			if((unsigned long)statbuf.st_mtime<but)ok=1;
		} else if(after&&before){
			//if(((unsigned long)statbuf.st_mtime>aut)&&((unsigned long)statbuf.st_mtime<but))ok=1;
			if(((unsigned long)statbuf.st_mtime>=aut)&&((unsigned long)statbuf.st_mtime<but))ok=1;
		} else {
			fprintf(stderr,"after/before Arya....\n");
			exit(-1);
		}
	} else ok=1;
	if(ok==0)continue;

		if(v>2){
			fprintf(stdout,"%10lu",statbuf.st_mtime);
			fprintf(stdout," ");
		}
		t=statbuf.st_mtime;
		tms=*gmtime(&t);
		if(v>1){
			fprintf(stdout,"%4d/%02d/%02d %02d:%02d:%02d",
				tms.tm_year+1900,tms.tm_mon+1,tms.tm_mday,
				tms.tm_hour,tms.tm_min,tms.tm_sec);
			fprintf(stdout," ");
		}

		link=0;
		reg=0;
		dir=0;
		device=0;
		special=0;
		unknown=0;

		if(S_ISDIR(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"d");
			dir=1;
		} else if(S_ISREG(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"-");
			reg=1;
		} else if(S_ISBLK(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"b");
			device=1;
		} else if(S_ISCHR(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"c");
			device=1;
		} else if(S_ISLNK(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"l");
			link=1;
		} else if(S_ISFIFO(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"F");
			special=1;
		} else if(S_ISSOCK(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"S");
			special=1;
		} else if(S_ISNAM(statbuf.st_mode)){
			if(v>1)fprintf(stdout,"N");
			special=1;
		} else {
			//if(v>1)fprintf(stdout,"X");
			if(v>1)fprintf(stdout,"%1X",(statbuf.st_mode&0xF000)>>12);
			unknown=1;
		}
#if 0
		if(v>2){
			fprintf(stdout," ");
			//fprintf(stdout,"%04X",statbuf.st_mode)
			fprintf(stdout,"%1X",(statbuf.st_mode&0xF000)>>12);
			fprintf(stdout,"%04o",(statbuf.st_mode&0007777));
			fprintf(stdout," ");
		}
#endif
		if(v>1){
		//XXXX//fprintf(stdout,"%04X",statbuf.st_mode&0x007777);
		fprintf(stdout,"%04o",statbuf.st_mode&007777);

			fprintf(stdout," ");

		fprintf(stdout,"%2u",statbuf.st_nlink);
			fprintf(stdout," ");
		//fprintf(stdout,"%5d/%5d",statbuf.st_uid,statbuf.st_gid);
		fprintf(stdout,"%d/%d",statbuf.st_uid,statbuf.st_gid);
			fprintf(stdout," ");
		fprintf(stdout,"%11lu",statbuf.st_size);
			fprintf(stdout," ");
		}

		if(v>0){
		fprintf(stdout,"%s",direntp->d_name);
		//fprintf(stdout,"%s\n",direntp->d_name);
		//if(strcmp(direntp->d_name,fnext)!=0)continue;
		//t=direntp->d_stat.st_ftime;
		}
		if(v>1){
		  if(dir)fprintf(stdout,"/");
		  if(special)fprintf(stdout,"#");
		  if(link)fprintf(stdout,"@");
		  if(unknown)fprintf(stdout," ??");
		  if(link){
			//XXlinklen=readlink(direntp->d_name,linkname,PATH_MAX);
			linklen=readlink(file,linkname,PATH_MAX);
			if(linklen==-1){
				perror(file);
			}else{
				linkname[linklen]='\0';
				printf(" -> %s",linkname);

				if(linkname[0]=='/'){
					strcpy(lfile,linkname);
				} else {
					strcpy(lfile,fullpath);
					if(fullpath[strlen(fullpath)-1]!='/')strcat(lfile,"/");
					strcat(lfile,linkname);
				}
				if(xxx==0){
					//fprintf(stdout,"--stat %s--",lfile);
				    if(stat(lfile,&statlbuf)!=0){
					//perror("stat:");
					//fprintf(stderr,"stat %s failed\n",file);
					//exit(-1);
						fprintf(stdout," ??");
				    } else {
					llink=0;
					lreg=0;
					ldir=0;
					ldevice=0;
					lspecial=0;
					lunknown=0;
			
					if(S_ISDIR(statlbuf.st_mode)){
						ldir=1;
						fprintf(stdout,"/");
					} else if(S_ISREG(statlbuf.st_mode)){
						lreg=1;
					} else if(S_ISBLK(statlbuf.st_mode)){
						ldevice=1;
					} else if(S_ISCHR(statlbuf.st_mode)){
						ldevice=1;
					} else if(S_ISLNK(statlbuf.st_mode)){
						llink=1;
						fprintf(stdout,"@");
					} else if(S_ISFIFO(statlbuf.st_mode)){
						lspecial=1;
						fprintf(stdout,"#");
					} else if(S_ISSOCK(statlbuf.st_mode)){
						lspecial=1;
						fprintf(stdout,"#");
					} else if(S_ISNAM(statlbuf.st_mode)){
						lspecial=1;
						fprintf(stdout,"#");
					} else {
						lunknown=1;
						fprintf(stdout,"??");
					}
				    }
				}

			}
		  }
		}
		if(v>0)fprintf(stdout,"\n");
		//if(w>2)fprintf(stdout,"file=%s\n",direntp->d_name);
		//if(w>2)fprintf(stdout,"fullpath=%s\n",file);

		//if(w>2)fprintf(stdout,"org-cmd is \"%s\"\n",cmd);

		if(absolute_path){
			strcpy(realcmd,rep_at_file(cmd,keychr,file));
		} else {
			strcpy(realcmd,rep_at_file(cmd,keychr,direntp->d_name));
		}

fflush(stdout);
		//if(w>1)fprintf(stdout,"realcmd is \"%s\"\n",realcmd);
		//if(w>1)fprintf(stdout,"wss: \"%s\"\n",realcmd);
		if(w>1)fprintf(stderr,"wss: \"%s\"\n",realcmd);

fflush(stderr);
		chdir(fullpath);
		rc = system(realcmd);
		chdir(pwd);
fflush(stdout);
fflush(stderr);
		if (rc == -1) {
			fprintf(stderr,"system: shell could not be run\n");
		} else {
		    if(w>0){
			rc2=WEXITSTATUS(rc);
			if(rc2){
			  //fprintf(stderr,"result of the command is %d\n",WEXITSTATUS(rc));
			  //fprintf(stderr,"wss: Error %d\n",rc2);
			  fprintf(stderr,"wss: Error \'%s\' returned %d\n",realcmd,rc2);
			  //fprintf(stderr,"wss: Error \'%s\' returned %d. Continue?  (y/n)",realcmd,rc2);
			}
		    }
		}
fflush(stdout);
fflush(stderr);

	}
	closedir(dirp);

	exit(0);
}

char *rep_at_file(char *orgcmd,char keychr,char *rep){
	char realcmd[1024]="";
	char keystr[2];
	char tmp0[1024];
	char tmp1[1024];
	char *where;
	int i;

	keystr[0]=keychr;
	keystr[1]='\0';

	//printf("org-cmd is %s\n",orgcmd);

	strcpy(tmp0,orgcmd);
	where=strchr(tmp0,keychr);
	while(where!=NULL){
		strcpy(tmp1,where);
		tmp0[strcspn(tmp0,keystr)]='\0';
		strcat(realcmd,tmp0);
		strcat(realcmd,rep);
		//XX//strcpy(tmp0,where);
		for(i=1;i<strlen(tmp1)+1;i++){
			tmp0[i-1]=tmp1[i];
		}
		where=strchr(tmp0,keychr);
	}
		strcat(realcmd,tmp0);
	//printf("realcmd is %s\n",realcmd);

	return realcmd;

}

