#ifdef __USAGE
%C - convert yyyy mo dy to weekday
%C [-n|-c|-C] yyyy mo dy [hr [mn [sc]]]
	yyyy : 4 digit year		(1900<=yyyy<=2100)
	 mo  : month			(1,2,3,...,12)
	 dy  : day			(1,2,3,...,31)
	 hr,mn,sc : will be just ignored if any
	-n : numerical expression (0=Sun, 1=Mon, ... , 6=Sat)
	-c : 3-character expression (Sun, Mon, ... , Sat)	[Default]
	-C : full character expression (Sunday, Monday, ... , Saturday)
#endif

#include <stdlib.h>
#include <stdio.h>
//#include <string.h>
#include <time.h>
#include <unistd.h>	//for basename();

#define  SECS_PER_DAY 86400L /* number of seconds per day */

void usage(int argc,char *argv[]){
	fprintf(stderr,"%s - convert yyyy mo dy to unix time\n",basename(argv[0]));
	fprintf(stderr,"\t%s [-n|-c|-C] yyyy mo dy [hr [mn [sc]]]\n",basename(argv[0]));
	fprintf(stderr,"\t\tyyyy : 4 digit year\t\t(1900<=yyyy<=2100)\n");
	fprintf(stderr,"\t\t mo  : month\t\t\t(1,2,3,...,12)\n");
	fprintf(stderr,"\t\t dy  : day of month\t\t(1,2,3,...,31)\n");
	fprintf(stderr,"\t\t hr,mn,sc  : will be just ignored if any\n");
	fprintf(stderr,"\t\t-n : numerical expression (0=Sun, 1=Mon, ... , 6=Sat)\n");
	fprintf(stderr,"\t\t-c : 3-character expression (Sun, Mon, ... , Sat)  [Default]\n");
	fprintf(stderr,"\t\t-C : full character expression (Sunday, Monday, ... , Saturday)\n");
	exit(-1);
}

void main(int argc,char *argv[]){

//###################################################################
//#struct  tm {
//#        int  tm_sec;    /* seconds after the minute -- [0,61] */
//#        int  tm_min;    /* minutes after the hour   -- [0,59] */
//#        int  tm_hour;   /* hours after midnight     -- [0,23] */
//#        int  tm_mday;   /* day of the month         -- [1,31] */
//#        int  tm_mon;    /* months since January     -- [0,11] */
//#        int  tm_year;   /* years since 1900                   */
//#        int  tm_wday;   /* days since Sunday        -- [0,6]  */
//#        int  tm_yday;   /* days since January 1     -- [0,365]*/
//#        int  tm_isdst;  /* Daylight Savings Time flag */
//#};
//###################################################################
struct tm tms;
int yr,mo,dy,hr,mn,sc,yd,wd,lp,yd2;
unsigned long yrsec,yrtmp,leap,tv_sec;
int bad=0;
int mode=-1;
int optflag=0;
static const char *weekday[]={
"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
static const char *weekdays[]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};

if(argc<2){
	usage(argc,argv);
}

if(strcmp(argv[1],"-n")==0)mode=0;
if(strcmp(argv[1],"-c")==0)mode=1;
if(strcmp(argv[1],"-C")==0)mode=2;
if(mode>-1){
	optflag=1;
} else mode=1;	// default mode
if(argc<4+optflag){
	usage(argc,argv);
}

yr=atoi(argv[1+optflag]);	// must be 4 digit!!!
mo=atoi(argv[2+optflag]);
dy=atoi(argv[3+optflag]);
//hr=atoi(argv[4+optflag]);
//mn=atoi(argv[5+optflag]);
//sc=atoi(argv[6+optflag]);

if((yr<1900)||(yr>2100)){
	fprintf(stderr,"month must be between 1900 and 2100\n");
	usage(argc,argv);
}
if((mo<1)||(mo>12)){
	fprintf(stderr,"month must be among 1,2,3,...,12\n");
	usage(argc,argv);
}
if((dy<1)||(dy>31)){
	fprintf(stderr,"day must be among 1,2,3,...,31\n");
	usage(argc,argv);
}
if(((mo==1)||(mo==3)||(mo==5)||(mo==7)||(mo==8)||(mo==10)||(mo==12))&&(dy>31)){
	fprintf(stderr,"day must be <= 31 for month=%d\n",mo);
	usage(argc,argv);
}
if(((mo==4)||(mo==6)||(mo==9)||(mo==11))&&(dy>30)){
	fprintf(stderr,"day must be <= 30 for month=%d\n",mo);
	usage(argc,argv);
}
	lp=(yr%4==0);
	if ( yr % 100  == 0 ) lp=0;
	if ( yr % 400  == 0 ) lp=1;
if((mo==2)&&(dy>28+lp)){
	fprintf(stderr,"day must be <= %d for year=%d,month=%d\n",28+lp,yr,mo);
	usage(argc,argv);
}

//if((yr>=1900)&&(yr=<2100)){
	tms.tm_year=yr-1900;
	tms.tm_mon=mo-1;
	tms.tm_mday=dy;
	tms.tm_hour=0;
	tms.tm_min=0;
	tms.tm_sec=0;
	tms.tm_isdst=0;

	tv_sec=mktime(&tms);

	//wd=tms.tm_wday+1;
	wd=tms.tm_wday;
	if(mode==0){
		fprintf(stdout,"%d\n",wd);
	}else if(mode==2){
		fprintf(stdout,"%s\n",weekday[wd]);
	}else{
		fprintf(stdout,"%s\n",weekdays[wd]);
	}
	exit(0);

	//yd=tms.tm_yday+1;
	//yd2=md2yd(yr,mo,dy);	// Jan. 1 = 1 (not 0!!!)
	//if(yd2==yd){
	//	fprintf(stdout,"%d\n",yd);
	//} else {
	//	fprintf(stdout,"%d\n",yd);
	//	fprintf(stderr,"%d (by mktime) != %d (my md2yd)\n",yd,yd2);
	//	fprintf(stderr,"Abnormal result...\n");
	//	exit(-1);
	//}
//} else {
//	//yd=md2yd(yr,mo,dy);	// Jan. 1 = 1 (not 0!!!)
//	//fprintf(stdout,"%d\n",yd);
//	usage(argc,argv);
//}
exit(0);

}

int md2yd(int yr,int mo,int dy){
/* a small function to calculate
 *	year day(total day) from year, month, day
 *	(1/1=1)
 */
	int leap,yd;
	//XXXXleap=((yr % 4) ? 1 : 0);
	leap=(yr%4==0);
	if ( yr % 100  == 0 ) leap=0;
	if ( yr % 400  == 0 ) leap=1;
	if ( mo <   0 )yd=0;
	if ( mo >  12 )yd=0;
	if ( mo ==  0 )yd=0;
	if ( mo ==  1 )yd=dy;
	if ( mo ==  2 )yd=dy+31;
	if ( mo ==  3 )yd=dy+leap+59;
	if ( mo ==  4 )yd=dy+leap+90;
	if ( mo ==  5 )yd=dy+leap+120;
	if ( mo ==  6 )yd=dy+leap+151;
	if ( mo ==  7 )yd=dy+leap+181;
	if ( mo ==  8 )yd=dy+leap+212;
	if ( mo ==  9 )yd=dy+leap+243;
	if ( mo == 10 )yd=dy+leap+273;
	if ( mo == 11 )yd=dy+leap+304;
	if ( mo == 12 )yd=dy+leap+334;

	//printf("yr=%d,mo=%d,dy=%d => yd=%d\n",yr,mo,dy,yd);

	return yd;
}

void monday(int *yr,int *mon,int *mday,int *yday) {
/* a small function to calculate
 *	mon (1-12), mday (1-31)  from  yr,yday(Jan.1=1)
 *				by Sessai@NIPR
 */

        int leap;

	leap=(*yr%4==0);
	if ( *yr % 100  == 0 ) leap=0;
	if ( *yr % 400  == 0 ) leap=1;

	if     (*yday <  1)     {*mon= 0;*mday=0;}
	else if(*yday<= 31)     {*mon= 1;*mday=*yday;}
	else if(*yday<= 59+leap){*mon= 2;*mday=*yday- 31;}
	else if(*yday<= 90+leap){*mon= 3;*mday=*yday- 59-leap;}
	else if(*yday<=120+leap){*mon= 4;*mday=*yday- 90-leap;}
	else if(*yday<=151+leap){*mon= 5;*mday=*yday-120-leap;}
	else if(*yday<=181+leap){*mon= 6;*mday=*yday-151-leap;}
	else if(*yday<=212+leap){*mon= 7;*mday=*yday-181-leap;}
	else if(*yday<=243+leap){*mon= 8;*mday=*yday-212-leap;}
	else if(*yday<=273+leap){*mon= 9;*mday=*yday-243-leap;}
	else if(*yday<=304+leap){*mon=10;*mday=*yday-273-leap;}
	else if(*yday<=334+leap){*mon=11;*mday=*yday-304-leap;}
	else if(*yday<=365+leap){*mon=12;*mday=*yday-334-leap;}
	else                    {*mon=0;*mday=0;}
        return;
}

