#ifdef __USAGE
%C - convert yyyy mo dy hr mn sc to unix time(seconds since Jan.1, 1970)
%C yyyy mo dy [hr [mn [sc]]]
	yyyy : 4 digit year		(1970<=yyyy<=2100))
	 mo  : month			(1,2,3,...,12)
	 dy  : day			(1,2,3,...,31)
	 hr  : hour			(0,1,2,...,23) (0 if omitted)
	 mn  : minute		(0,1,2,...,59) (0 if omitted)
	 sc  : second		(0,1,2,...,59) (0 if omitted)
#endif

#include <stdlib.h>
#include <stdio.h>
//#include <string.h>
#include <time.h>
#include <unistd.h>	//for basename();

#define  SECS_PER_DAY 86400L /* number of seconds per day */

void usage(int argc,char *argv[]){
	fprintf(stderr,"%s - convert yyyy mo dy to unix time(sec since 1970/1/1)\n",basename(argv[0]));
	fprintf(stderr,"\t%s yyyy mo dy [hr [mn [sc]]]\n",basename(argv[0]));
	fprintf(stderr,"\t\tyyyy : 4 digit year\t(1970<=yyyy<=2100)\n");
	fprintf(stderr,"\t\t mo  : month\t\t(1,2,3,...,12)\n");
	fprintf(stderr,"\t\t dy  : day of month\t(1,2,3,...,31)\n");
	fprintf(stderr,"\t\t hr  : hour\t\t(0,1,2,...,23) (0 if omitted)\n");
	fprintf(stderr,"\t\t mn  : minute\t\t(0,1,2,...,59) (0 if omitted)\n");
	fprintf(stderr,"\t\t sc  : second\t\t(0,1,2,...,59) (0 if omitted)\n");
	exit(-1);
}

void main(int argc,char *argv[]){

//###################################################################
//#struct  tm {
//#        int  tm_sec;    /* seconds after the minute -- [0,61] */
//#        int  tm_min;    /* minutes after the hour   -- [0,59] */
//#        int  tm_hour;   /* hours after midnight     -- [0,23] */
//#        int  tm_mday;   /* day of the month         -- [1,31] */
//#        int  tm_mon;    /* months since January     -- [0,11] */
//#        int  tm_year;   /* years since 1900                   */
//#        int  tm_wday;   /* days since Sunday        -- [0,6]  */
//#        int  tm_yday;   /* days since January 1     -- [0,365]*/
//#        int  tm_isdst;  /* Daylight Savings Time flag */
//#};
//###################################################################
	struct tm tms;
	int yr,mo,dy,hr=0,mn=0,sc=0,lp,yd;
	unsigned long yrsec,yrtmp,leap,tv_sec,tv_sec2;

	if(argc<4)usage(argc,argv);

	yr=atoi(argv[1]);	// must be 4 digit!!!
	mo=atoi(argv[2]);
	dy=atoi(argv[3]);
	if(argc>4)hr=atoi(argv[4]);
	if(argc>5)mn=atoi(argv[5]);
	if(argc>6)sc=atoi(argv[6]);

	lp=(yr%4==0);
	if ( yr % 100  == 0 ) lp=0;
	if ( yr % 400  == 0 ) lp=1;

	if((yr<1970)||(yr>2100)){
		fprintf(stderr,"month must be between 1970 & 2100\n");
		usage(argc,argv);
	}
	if((mo<1)||(mo>12)){
		fprintf(stderr,"month must be among 1,2,3,...,12\n");
		usage(argc,argv);
	}
	if((dy<1)||(dy>31)){
		fprintf(stderr,"day must be among 1,2,3,...,31\n");
		usage(argc,argv);
	}
	if(((mo==1)||(mo==3)||(mo==5)||(mo==7)||(mo==8)||(mo==10)||(mo==12))
		&&(dy>31)){
		fprintf(stderr,"day must be <= 31 for month=%d\n",mo);
		usage(argc,argv);
	}
	if(((mo==4)||(mo==6)||(mo==9)||(mo==11))&&(dy>30)){
		fprintf(stderr,"day must be <= 30 for month=%d\n",mo);
		usage(argc,argv);
	}
	if((mo==2)&&(dy>28+lp)){
		fprintf(stderr,"day must be <= %d for year=%d,month=%d\n",
			28+lp,yr,mo);
		usage(argc,argv);
	}
	if((hr<0)||(hr>23)){
		fprintf(stderr,"hour must be among 0,1,2,3,...,23\n");
		usage(argc,argv);
	}
	if((mn<0)||(mn>60)){
		fprintf(stderr,"minute must be among 0,1,2,3,...,59(,60)\n");
		usage(argc,argv);
	}
	if((sc<0)||(sc>60)){
		fprintf(stderr,"second must be among 0,1,2,3,...,59(,60)\n");
		usage(argc,argv);
	}

	tms.tm_year=yr-1900;
	tms.tm_mon=mo-1;
	tms.tm_mday=dy;
	tms.tm_hour=hr;
	tms.tm_min=mn;
	tms.tm_sec=sc;
	tms.tm_isdst=0;

	tv_sec=mktime(&tms);

	//fprintf(stdout,"%lu\n",tv_sec);


	yd=md2yd(yr,mo,dy);	// Jan. 1 = 1 (not 0!!!)
	yrsec=(unsigned long)(yd-1)*SECS_PER_DAY
		+(unsigned long)hr*3600L+(unsigned long)mn*60L
		+(unsigned long)sc;

	yrtmp=yr-1970L;
	// leap=(yrtmp/4L)+((yrtmp % 4L) ? 1L : 0L);  	// Rob's original
	leap=((yrtmp-10)/4L)+(((yrtmp-10) % 4L) ? 1L : 0L) + 2L;  //Sessai fix!
	//XXXXX!!!!!!!!if(yr>=2100)leap--;
	if(yr>2100)leap--;
	//if(yr>2200)leap--;
	//if(yr>2300)leap--;
	//if(yr>2500)leap--;
	//if(yr>2600)leap--;
	//if(yr>2700)leap--;
	//if(yr>2900)leap--;

	tv_sec2=(leap+(365L*yrtmp))*SECS_PER_DAY+yrsec;

	//printf("%lu\n",tv_sec2);
	if(tv_sec2 == tv_sec){
		fprintf(stdout,"%lu\n",tv_sec);
	} else {
		fprintf(stdout,"%lu\n",tv_sec);
		fprintf(stderr,"%lu(by mktime) != %lu(by mine)\n",
			tv_sec,tv_sec2);
		fprintf(stderr,"Abnormal result...\n");
		exit(-1);
	}

	exit(0);

}

int md2yd(int yr,int mo,int dy){
/* a small function to calculate
 *	year day(total day) from year, month, day
 *	(1/1=1)
 */
	int leap,yd;
	//XXXXleap=((yr % 4) ? 1 : 0);
	leap=(yr%4==0);
	if ( yr % 100  == 0 ) leap=0;
	if ( yr % 400  == 0 ) leap=1;
	if ( mo <   0 )yd=0;
	if ( mo >  12 )yd=0;
	if ( mo ==  0 )yd=0;
	if ( mo ==  1 )yd=dy;
	if ( mo ==  2 )yd=dy+31;
	if ( mo ==  3 )yd=dy+leap+59;
	if ( mo ==  4 )yd=dy+leap+90;
	if ( mo ==  5 )yd=dy+leap+120;
	if ( mo ==  6 )yd=dy+leap+151;
	if ( mo ==  7 )yd=dy+leap+181;
	if ( mo ==  8 )yd=dy+leap+212;
	if ( mo ==  9 )yd=dy+leap+243;
	if ( mo == 10 )yd=dy+leap+273;
	if ( mo == 11 )yd=dy+leap+304;
	if ( mo == 12 )yd=dy+leap+334;

	//printf("yr=%d,mo=%d,dy=%d => yd=%d\n",yr,mo,dy,yd);

	return yd;
}

void monday(int *yr,int *mon,int *mday,int *yday) {
/* a small function to calculate
 *	mon (1-12), mday (1-31)  from  yr,yday(Jan.1=1)
 *				by Sessai@NIPR
 */

        int leap;

	leap=(*yr%4==0);
	if ( *yr % 100  == 0 ) leap=0;
	if ( *yr % 400  == 0 ) leap=1;

	if     (*yday <  1)     {*mon= 0;*mday=0;}
	else if(*yday<= 31)     {*mon= 1;*mday=*yday;}
	else if(*yday<= 59+leap){*mon= 2;*mday=*yday- 31;}
	else if(*yday<= 90+leap){*mon= 3;*mday=*yday- 59-leap;}
	else if(*yday<=120+leap){*mon= 4;*mday=*yday- 90-leap;}
	else if(*yday<=151+leap){*mon= 5;*mday=*yday-120-leap;}
	else if(*yday<=181+leap){*mon= 6;*mday=*yday-151-leap;}
	else if(*yday<=212+leap){*mon= 7;*mday=*yday-181-leap;}
	else if(*yday<=243+leap){*mon= 8;*mday=*yday-212-leap;}
	else if(*yday<=273+leap){*mon= 9;*mday=*yday-243-leap;}
	else if(*yday<=304+leap){*mon=10;*mday=*yday-273-leap;}
	else if(*yday<=334+leap){*mon=11;*mday=*yday-304-leap;}
	else if(*yday<=365+leap){*mon=12;*mday=*yday-334-leap;}
	else                    {*mon=0;*mday=0;}
        return;
}

