#ifdef __USAGE
%C - calculate yearsec from YYYY MM DD hh mm ss
%C YYYY MM DD [hh [mm [ss]]]
	YYYY : 4-digit year
	 MM  : month
	 DD  : day of month
	 hh  : hour	(0 if omitted)
	 mm  : minute	(0 if omitted)
	 ss  : second	(0 if omitted)
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>	//for basename();

void usage(int argc, char *argv[]){
	fprintf(stderr,"%s - calculate yearsec from YYYY MM DD hh mm ss\n",basename(argv[0]));
	fprintf(stderr,"%s YYYY MM DD [hh [mm [ss]]]\n",basename(argv[0]));
	fprintf(stderr,"\thh,mm,ss will be 0 if omitted\n");
	exit(-1);
}

void main(int argc, char *argv[]){
	int yr,mo,dy,hr=0,mn=0,sc=0,leap,yd;
	struct tm tms;
	unsigned long yrsec;

	if(argc<4)usage(argc,argv);
	yr=atoi(argv[1]);
	mo=atoi(argv[2]);
	dy=atoi(argv[3]);
	if(argc>4)hr=atoi(argv[4]);
	if(argc>5)mn=atoi(argv[5]);
	if(argc>6)sc=atoi(argv[6]);
	//XXXXleap=((yr % 4) ? 1 : 0);
	leap=(yr%4==0);
	if ( yr % 100  == 0 ) leap=0;
	if ( yr % 400  == 0 ) leap=1;

	if((mo<1)||(mo>12)){
		fprintf(stderr,"month must be among 1,2,3,...,12\n");
		usage(argc,argv);
	}
	if((dy<1)||(dy>31)){
		fprintf(stderr,"day must be among 1,2,3,...,31\n");
		usage(argc,argv);
	}
	if(((mo==1)||(mo==3)||(mo==5)||(mo==7)||(mo==8)||(mo==10)||(mo==12))
		&&(dy>31)){
		fprintf(stderr,"day must be <= 31 for month=%d\n",mo);
		usage(argc,argv);
	}
	if(((mo==4)||(mo==6)||(mo==9)||(mo==11))&&(dy>30)){
		fprintf(stderr,"day must be <= 30 for month=%d\n",mo);
		usage(argc,argv);
	}
	if((mo==2)&&(dy>28+leap)){
		fprintf(stderr,"day must be <= %d for year=%d,month=%d\n",
			28+leap,yr,mo);
		usage(argc,argv);
	}

	// not use mktime to be safe for any year...
	yd=md2yd(yr,mo,dy);	// Jan. 1 = 1 (not 0!!!)

	yd--;			// Jan. 1 = 0 !!!

	yrsec=(unsigned long)yd*86400L+(unsigned long)hr*3600L
		+(unsigned long)mn*60L+(unsigned long)sc;
	fprintf(stdout,"%lu\n",yrsec);

	exit(0);
}

//###################################################################
//#struct  tm {
//#        int  tm_sec;    /* seconds after the minute -- [0,61] */
//#        int  tm_min;    /* minutes after the hour   -- [0,59] */
//#        int  tm_hour;   /* hours after midnight     -- [0,23] */
//#        int  tm_mday;   /* day of the month         -- [1,31] */
//#        int  tm_mon;    /* months since January     -- [0,11] */
//#        int  tm_year;   /* years since 1900                   */
//#        int  tm_wday;   /* days since Sunday        -- [0,6]  */
//#        int  tm_yday;   /* days since January 1     -- [0,365]*/
//#        int  tm_isdst;  /* Daylight Savings Time flag */
//#};
//###################################################################

int md2yd(int yr,int mo,int dy){
/* a small function to calculate
 *	year day(total day) from year, month, day
 *	(1/1=1)
 */
	int leap,yd;
	//XXXXleap=((yr % 4) ? 1 : 0);
	leap=(yr%4==0);
	if ( yr % 100  == 0 ) leap=0;
	if ( yr % 400  == 0 ) leap=1;
	if ( mo <   0 )yd=0;
	if ( mo >  12 )yd=0;
	if ( mo ==  0 )yd=0;
	if ( mo ==  1 )yd=dy;
	if ( mo ==  2 )yd=dy+31;
	if ( mo ==  3 )yd=dy+leap+59;
	if ( mo ==  4 )yd=dy+leap+90;
	if ( mo ==  5 )yd=dy+leap+120;
	if ( mo ==  6 )yd=dy+leap+151;
	if ( mo ==  7 )yd=dy+leap+181;
	if ( mo ==  8 )yd=dy+leap+212;
	if ( mo ==  9 )yd=dy+leap+243;
	if ( mo == 10 )yd=dy+leap+273;
	if ( mo == 11 )yd=dy+leap+304;
	if ( mo == 12 )yd=dy+leap+334;

	//printf("yr=%d,mo=%d,dy=%d => yd=%d\n",yr,mo,dy,yd);

	return yd;
}

void monday(int *yr,int *mon,int *mday,int *yday) {
/* a small function to calculate
 *	mon (1-12), mday (1-31)  from  yr,yday(Jan.1=1)
 *				by Sessai@NIPR
 */

        int leap;

	leap=(*yr%4==0);
	if ( *yr % 100  == 0 ) leap=0;
	if ( *yr % 400  == 0 ) leap=1;

	if     (*yday <  1)     {*mon= 0;*mday=0;}
	else if(*yday<= 31)     {*mon= 1;*mday=*yday;}
	else if(*yday<= 59+leap){*mon= 2;*mday=*yday- 31;}
	else if(*yday<= 90+leap){*mon= 3;*mday=*yday- 59-leap;}
	else if(*yday<=120+leap){*mon= 4;*mday=*yday- 90-leap;}
	else if(*yday<=151+leap){*mon= 5;*mday=*yday-120-leap;}
	else if(*yday<=181+leap){*mon= 6;*mday=*yday-151-leap;}
	else if(*yday<=212+leap){*mon= 7;*mday=*yday-181-leap;}
	else if(*yday<=243+leap){*mon= 8;*mday=*yday-212-leap;}
	else if(*yday<=273+leap){*mon= 9;*mday=*yday-243-leap;}
	else if(*yday<=304+leap){*mon=10;*mday=*yday-273-leap;}
	else if(*yday<=334+leap){*mon=11;*mday=*yday-304-leap;}
	else if(*yday<=365+leap){*mon=12;*mday=*yday-334-leap;}
	else                    {*mon=0;*mday=0;}
        return;
}

