#ifdef __USAGE
%C - convert year and yearsec to year, mon, day, hour, min. and sec.
%C [-c|-C|-s|-S] [-d|-t] yyyy ysec
	yyyy : 4 digit year
	ysec : seconds of year
    output will be "YYYY MM DD hh mm ss" (if no options specified)
               "YYYYMMDD hhmmss"     (if (only) "-c" specified)
               "YYYYMMDDhhmmss"      (if (only) "-C" specified)
               "YYYY/MM/DD hh:mm:ss" (if (only) "-s" or "-S" specified)
               "YYYY MM DD"          (if (only) "-d" specified)
               "hh mm ss"            (if (only) "-t" specified)
               "YYYYMMDD"            (if "-c -d" or "-C -d" specified)
               "hhmmss"              (if "-c -t" or "-C -t" specified)
               "YYYY/MM/DD"          (if "-s -d" or "-S -d" specified)
               "hh:mm:ss"            (if "-s -t" or "-S -t" specified)
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>	//for basename();

#define  SECS_PER_DAY 86400L /* number of seconds per day */

#if 0
void usage(int argc,char *argv[]){
	fprintf(stderr,"%s - convert year and yearsec to yyyy mo dy hr mn sc\n",basename(argv[0]));
	fprintf(stderr,"\t%s [-c|-C|-s|-S] [-d|-t] yyyy ysec\n",basename(argv[0]));
	fprintf(stderr,"\t\tyyyy : 4 digit year\n");
	fprintf(stderr,"\t\tysec : seconds of year\n");
	fprintf(stderr,"\toutput will be YYYY MM DD hh mm ss (if no options specified)\n");
	fprintf(stderr,"\t               YYYYMMDD hhmmss     (if (only) \"-c\" specified)\n");
	fprintf(stderr,"\t               YYYYMMDDhhmmss      (if (only) \"-C\" specified)\n");
	fprintf(stderr,"\t               YYYY/MM/DD hh:mm:ss (if (only) \"-s\" or \"-S\" specified)\n");
	fprintf(stderr,"\t               YYYY MM DD          (if (only) \"-d\" specified)\n");
	fprintf(stderr,"\t               hh mm ss            (if (only) \"-t\" specified)\n");
	fprintf(stderr,"\t               YYYYMMDD            (if \"-c -d\" or \"-C -d\" specified)\n");
	fprintf(stderr,"\t               hhmmss              (if \"-c -t\" or \"-C -t\" specified)\n");
	fprintf(stderr,"\t               YYYY/MM/DD          (if \"-s -d\" or \"-S -d\" specified)\n");
	fprintf(stderr,"\t               hh:mm:ss            (if \"-s -t\" or \"-S -t\" specified)\n");
	exit(-1);
}
#else
void usage(int argc,char *argv[]){
	char cmd[48];
	strcpy(cmd,"use ");
	strcat(cmd,argv[0]);
	strcat(cmd," 1>&2");
	system(cmd);
	exit(-1);
}
#endif


void main(int argc,char *argv[]){

	int yr,mo,dy,hr,mn,sc,lp,yd;
	unsigned long ys,ds;
	int mode=0;	// 0 for date&time, 1 for date only, & 2 for time only
	int cmp=0;	// 1 for -c, 2 for -C, -1 for -s/-S 0 for default
	int optflagm=0; // 1 if -d or -t specified
	int optflagc=0; // 1 if -c, -C, -s or -S specified

	struct tm tms;
	unsigned long tv_sec;

	if(argc==1)usage(argc,argv);

	if(strcmp(argv[1],"-c")==0){cmp=1;optflagc=1;}
	if(strcmp(argv[1],"-C")==0){cmp=2;optflagc=1;}
	if(strcmp(argv[1],"-s")==0){cmp=-1;optflagc=1;}
	if(strcmp(argv[1],"-S")==0){cmp=-1;optflagc=1;}

	if(strcmp(argv[1+optflagc],"-d")==0){mode=1;optflagm=1;}
	if(strcmp(argv[1+optflagc],"-t")==0){mode=2;optflagm=1;}

	//if(argc<3+mode)usage(argc,argv);
	if(argc != 3+optflagc+optflagm)usage(argc,argv);

	yr=atoi(argv[1+optflagc+optflagm]);	// must be 4 digit!!!
	ys=atol(argv[2+optflagc+optflagm]);

	lp=(yr%4==0);
	if ( yr % 100  == 0 ) lp=0;
	if ( yr % 400  == 0 ) lp=1;

	if((ys<0L)||(ys>=((unsigned long)(365+lp))*86400L)){
		fprintf(stderr,"ysec must be between 1 and %lu for year=%d\n",
			(unsigned long)(365+lp)*86400L-1L,yr);
		fprintf(stderr,"given ysec = %lu (%ld)\n",ys,ys);
		usage(argc,argv);
	}

	yd=(int)(ys/86400L);	// Jan.1 = 0 here...
	ds=ys%86400L;		//ds=ys-(unsigned long)yd*86400L;
	hr=(int)(ds/3600L);
	mn= ((int)(ds%3600L))/60;
	sc=(int)(ds%60L);

	yd++;			// Jan.1 = 1 here...
	monday(&yr,&mo,&dy,&yd);	// yd=1 means Jan.1

	if(mode==1){
	    if(cmp>0)		fprintf(stdout,"%04d%02d%02d\n",yr,mo,dy);
	    else if(cmp<0)	fprintf(stdout,"%04d/%02d/%02d\n",yr,mo,dy);
	    else		fprintf(stdout,"%d %d %d\n",yr,mo,dy);
	} else if(mode==2){
	    if(cmp>0)		fprintf(stdout,"%02d%02d%02d\n",hr,mn,sc);
	    else if(cmp<0)	fprintf(stdout,"%02d:%02d:%02d\n",hr,mn,sc);
	    else		fprintf(stdout,"%d %d %d\n",hr,mn,sc);
	} else {
	    if(cmp==2)
		fprintf(stdout,"%04d%02d%02d%02d%02d%02d\n",yr,mo,dy,hr,mn,sc);
	    else if(cmp==1)
		fprintf(stdout,"%04d%02d%02d %02d%02d%02d\n",yr,mo,dy,hr,mn,sc);
	    else if(cmp==-1)
		fprintf(stdout,"%04d/%02d/%02d %02d:%02d:%02d\n",yr,mo,dy,hr,mn,sc);
	    else
		fprintf(stdout,"%d %d %d %d %d %d\n",yr,mo,dy,hr,mn,sc);
	}

	exit(0);

}

int md2yd(int yr,int mo,int dy){
/* a small function to calculate
 *	year day(total day) from year, month, day
 *	(1/1=1)
 */
	int leap,yd;

	//XXXXleap=((yr % 4) ? 1 : 0);
	leap=(yr%4==0);
	if ( yr % 100  == 0 ) leap=0;
	if ( yr % 400  == 0 ) leap=1;

	if ( mo <   0 )yd=0;
	if ( mo >  12 )yd=0;
	if ( mo ==  0 )yd=0;
	if ( mo ==  1 )yd=dy;
	if ( mo ==  2 )yd=dy+31;
	if ( mo ==  3 )yd=dy+leap+59;
	if ( mo ==  4 )yd=dy+leap+90;
	if ( mo ==  5 )yd=dy+leap+120;
	if ( mo ==  6 )yd=dy+leap+151;
	if ( mo ==  7 )yd=dy+leap+181;
	if ( mo ==  8 )yd=dy+leap+212;
	if ( mo ==  9 )yd=dy+leap+243;
	if ( mo == 10 )yd=dy+leap+273;
	if ( mo == 11 )yd=dy+leap+304;
	if ( mo == 12 )yd=dy+leap+334;

	//printf("yr=%d,mo=%d,dy=%d => yd=%d\n",yr,mo,dy,yd);

	return yd;
}

void monday(int *yr,int *mon,int *mday,int *yday) {
/* a small function to calculate
 *	mon (1-12), mday (1-31)  from  yr,yday(Jan.1=1)
 *				by Sessai@NIPR
 */

        int leap;

	leap=(*yr%4==0);
	if ( *yr % 100  == 0 ) leap=0;
	if ( *yr % 400  == 0 ) leap=1;

	if     (*yday <  1)     {*mon= 0;*mday=0;}
	else if(*yday<= 31)     {*mon= 1;*mday=*yday;}
	else if(*yday<= 59+leap){*mon= 2;*mday=*yday- 31;}
	else if(*yday<= 90+leap){*mon= 3;*mday=*yday- 59-leap;}
	else if(*yday<=120+leap){*mon= 4;*mday=*yday- 90-leap;}
	else if(*yday<=151+leap){*mon= 5;*mday=*yday-120-leap;}
	else if(*yday<=181+leap){*mon= 6;*mday=*yday-151-leap;}
	else if(*yday<=212+leap){*mon= 7;*mday=*yday-181-leap;}
	else if(*yday<=243+leap){*mon= 8;*mday=*yday-212-leap;}
	else if(*yday<=273+leap){*mon= 9;*mday=*yday-243-leap;}
	else if(*yday<=304+leap){*mon=10;*mday=*yday-273-leap;}
	else if(*yday<=334+leap){*mon=11;*mday=*yday-304-leap;}
	else if(*yday<=365+leap){*mon=12;*mday=*yday-334-leap;}
	else                    {*mon=0;*mday=0;}
        return;
}

